package org.globus.transfer.reliable.service.cache;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.transfer.reliable.service.exception.RftException;
import commonj.timers.Timer;
import commonj.timers.TimerListener;
import commonj.timers.TimerManager;

public class Sweeper implements TimerListener
{
    private static Log logger = LogFactory.getLog(
        Sweeper.class.getName());

    private TimerManager timerManager;
    private ConnectionManager connectionManager;
    private Timer timer;
    private int sweeperInterval;
    
    public Sweeper(
        TimerManager      timerManager,
        ConnectionManager connectionManager,
        int               sweeperInterval)
    throws RftException
    {
        if (connectionManager == null)
        {
            throw new RftException(
                "Internal cache error: connectionManager is null " +
                "but must not be null");    
        }
        if (timerManager == null)
        {
            throw new RftException(
                "Internal cache error: timerManager is null " +
                "but must not be null");    
        }
        if (sweeperInterval < 0)
        {
            throw new RftException(
                "Configuration error: sweeperInterval < 0");    
        }
        
        this.connectionManager = connectionManager;
        this.timerManager = timerManager;
        this.sweeperInterval = sweeperInterval;
        this.timer = null;
    }
    
    /**
     * Called by the TimerManager (interface TimerListener)
     */
    public void timerExpired(Timer timer)
    {
        if (logger.isDebugEnabled()) {
            logger.debug("timer expired. removing expired connections.");
        }
        this.connectionManager.removeExpiredConnections();
        
        // Schedule the sweeper if the cache is not empty
        this.resetSweeper();
        if (this.connectionManager.getNumberCachedFDs() > 0) {
            this.scheduleSweeper();
        }
    }

    /**
     * Remove the timer
     */
    private synchronized void resetSweeper() 
    {
        this.timer = null;
    }
    
    /**
     * Re-schedule the timer if necessary
     */
    public synchronized void scheduleSweeper() 
    {
        if (this.timer == null)
        {
            this.timer = this.timerManager.schedule(
                this,this.sweeperInterval);
            if (logger.isDebugEnabled())
            {
                logger.debug("scheduling timer");
            }
        } else {
            if (logger.isDebugEnabled())
            {
                logger.debug("not scheduling timer");
            }
        }
    }
}